/*
 * filter.c: Filter class
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <stdlib.h>
#include <vdr/tools.h>

#include "math.h"
#include "defs.h"
#include "setup.h"
#include "filter.h"

static tRGBColorLongInt *mean_sums; // needed for the running mean value filter
static tRGBColor        *mean_values;
static tRGBColor        *filter_old;

cColFilter::cColFilter(unsigned int sum)
{
  cnt = sum;

  // Initialization
  mean_sums   = new tRGBColorLongInt[cnt];
  mean_values = new tRGBColor[cnt];
  filter_old  = new tRGBColor[cnt];
  ResetFilter();
}

cColFilter::~cColFilter()
{
  delete[] mean_sums;
  delete[] mean_values;
  delete[] filter_old;
}

void cColFilter::ResetFilter(void)
{
  // reset filter values
  for (unsigned int i=0; i<cnt; i++)
  {
    mean_sums[i].r   = mean_sums[i].g   = mean_sums[i].b   = 0;
    mean_values[i].r = mean_values[i].g = mean_values[i].b = 0;
    filter_old[i].r  = filter_old[i].g  = filter_old[i].b  = 0;
  }
}

void cColFilter::DoFilter(tRGBColor *col)
{
  // apply filter
  if (combined == SolarstormSetup.FilterMode)
  {
    for (unsigned int ch = 0; ch < cnt; ch++)
    {
      // calculate the mean-value filters
      mean_sums[ch].r += (long int)(col[ch].r - mean_values[ch].r); // red
      mean_values[ch].r = mean_sums[ch].r / ((long int)SolarstormSetup.FilterMeanLength);

      mean_sums[ch].g += (long int)(col[ch].g - mean_values[ch].g); // green
      mean_values[ch].g = mean_sums[ch].g / ((long int)SolarstormSetup.FilterMeanLength);

      mean_sums[ch].b += (long int)(col[ch].b - mean_values[ch].b); // blue
      mean_values[ch].b = mean_sums[ch].b / ((long int)SolarstormSetup.FilterMeanLength);

      // check, if there is a jump -> check if differences between actual values and filter values are too big
      long int dist; // distance between the two colors in the 3D RGB space
      dist = (mean_values[ch].r - col[ch].r)*(mean_values[ch].r - col[ch].r)+
             (mean_values[ch].g - col[ch].g)*(mean_values[ch].g - col[ch].g)+
             (mean_values[ch].b - col[ch].b)*(mean_values[ch].b - col[ch].b);
      dist = (long int)sqrt((double)dist);

      // compare calculated distance with the filter threshold
      if (dist > (long int)((double)SolarstormSetup.FilterMeanThreshold * 4.42f))
      {
        // filter jump detected -> set the long filters to the result of the short filters
        mean_values[ch] = col[ch];

        mean_sums[ch].r = col[ch].r * (SolarstormSetup.FilterMeanLength);
        mean_sums[ch].g = col[ch].g * (SolarstormSetup.FilterMeanLength);
        mean_sums[ch].b = col[ch].b * (SolarstormSetup.FilterMeanLength);
      }
      else
      {
        // apply an additional percent filter and return calculated values
        col[ch].r = (mean_values[ch].r * (100-SolarstormSetup.FilterPercentNew) + filter_old[ch].r * SolarstormSetup.FilterPercentNew) / 100;
        col[ch].g = (mean_values[ch].g * (100-SolarstormSetup.FilterPercentNew) + filter_old[ch].g * SolarstormSetup.FilterPercentNew) / 100;
        col[ch].b = (mean_values[ch].b * (100-SolarstormSetup.FilterPercentNew) + filter_old[ch].b * SolarstormSetup.FilterPercentNew) / 100;
      }
    }
    memcpy(filter_old, col, cnt * sizeof(tRGBColor));
  }
}
