/*
 * inputgrab.c: input from VDR-GrabImage()
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <stdio.h>
#include <string.h>
#include <stdint.h>

#include <vdr/device.h>
#include <vdr/plugin.h>
#include <vdr/tools.h>

#include "defs.h"
#include "calculation.h"
#include "input.h"
#include "inputgrab.h"

static int pnm_header_length;
static uint16_t width, height;
static uint32_t ImageSizeCalc;

cInputGrab::cInputGrab(uint16_t w, uint16_t h)
{
  width = w;
  height = h;

  // length of PNM header to remove
  char buf[32];
  snprintf(buf, sizeof(buf), "P6\n%d\n%d\n255\n", width, height);
  pnm_header_length = strlen(buf);
  ImageSizeCalc = 3 * width * height + pnm_header_length;
}

cInputGrab::~cInputGrab()
{
}

bool cInputGrab::Open(const char *param)
{
  return true;
}

bool cInputGrab::Close(void)
{
  return true;
}

bool cInputGrab::GetHSVImg(tHSVColor *hsv)
{
  unsigned char r, g, b;
  int ImageSize;

  // read picture
  uchar *Image = cDevice::PrimaryDevice()->GrabImage(ImageSize, false, 100, width, height);

  cCondWait::SleepMs(1);

  // calculate colors
  if (Image)
  {
    uchar *tmpImage = Image + pnm_header_length;

    if (ImageSize != ImageSizeCalc)
    {
      esyslog("Solarstorm: ImageSize != IMAGE_SIZE (%d)", ImageSize);
    }

    // generate HSV image
    for (int i = 0; i < (width*height); i++) // walk through map
    {
      // generate HSV pixel
      r = *tmpImage++;
      g = *tmpImage++;
      b = *tmpImage++;
      hsv[i] = RGB2HSV(r, g, b);
    }

    free(Image);

    return true;
  }
  else
  {
    return false;
  }
}
