/*
 * menu.c: Menus
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <vdr/plugin.h>
#include <vdr/remote.h>

#include "defs.h"
#include "setup.h"
#include "solarstorm.h"
#include "menu.h"

extern cPlugin *SolarstormPlugin;

// --- cMenuSetupMenu --------------------------------------------------------

#define STARTMODE           "Startmode"

cMenuSetupMenu::cMenuSetupMenu(void)
{
  useStartModeTexts[0] = tr("always on");
  useStartModeTexts[1] = tr("always off");
  useStartModeTexts[2] = tr("timer dependent");

  Set();
}

void cMenuSetupMenu::Set(void)
{
  int current = Current();
  Clear();

  Add(new cMenuEditStraItem(tr("Startmode"), &SolarstormSetup.StartMode, 3, useStartModeTexts));
  Add(new cOsdItem(tr("Mode settings..."), osUser3));
  Add(new cOsdItem(tr("Filter settings..."), osUser4));
  Add(new cOsdItem(tr("Channel settings..."), osUser5));
  Add(new cOsdItem(tr("Gamma settings..."), osUser6));
  Add(new cOsdItem(tr("Color calibration..."), osUser7));

  SetCurrent(Get(current));
  Display();
}

bool cMenuSetupMenu::SetupParse(const char *Name, const char *Value)
{
  if (!strcasecmp(Name, STARTMODE)) SolarstormSetup.StartMode = atoi(Value);
  else return false;

  return true;
}

void cMenuSetupMenu::Store(void)
{
  SetupStore(STARTMODE, SolarstormSetup.StartMode);
}

eOSState cMenuSetupMenu::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  switch (state)
  {
    case osUser3:
      return(AddSubMenu(new cMenuSetupMode));
    break;

    case osUser4:
      return(AddSubMenu(new cMenuSetupFilter));
    break;

    case osUser5:
      return(AddSubMenu(new cMenuSetupChannels));
    break;

    case osUser6:
      return(AddSubMenu(new cMenuSetupGamma));
    break;

    case osUser7:
         ((cPluginSolarstorm*)SolarstormPlugin)->DoWhiteCalibration();
         cRemote::CallPlugin("solarstorm");
         return osBack;
    break;

    case osUnknown:
         if ((Key == kOk) || (Key == kBack))
         {
           Store();
           state = osBack;
         }
    break;

    default:
    break;
  }
  return state;
}

// --- cMenuSetupMode --------------------------------------------------------

#define MODE                   "Mode"
#define ANALYSISDEPTH          "AnalysisDepth"
#define EDGEWEIGHTING          "EdgeWeighting"
#define MININPUTBRIGHTNESS     "MinInputBrightness"
#define MINOUTPUTBRIGHTNESS    "MinOutputBrightness"
#define BRIGHTNESSCORRECTION   "BrightnessCorrection"
#define HISTOGRAMANALYSISWIDTH "HistogramAnalysisWidth"
#define STATICCOLOR            "StaticColor"

cMenuSetupMode::cMenuSetupMode(void) : cOsdMenu(tr("Mode settings"), 25)
{
  useModeTexts[live_picture_hsv] = tr("live picture"); // generate colors out of live picture
  useModeTexts[static_color]     = tr("static color"); // select static color

  useStaticColorsTexts[black]   = tr("black");
  useStaticColorsTexts[white]   = tr("white");
  useStaticColorsTexts[red]     = tr("red");
  useStaticColorsTexts[green]   = tr("green");
  useStaticColorsTexts[blue]    = tr("blue");
  useStaticColorsTexts[yellow]  = tr("yellow");
  useStaticColorsTexts[cyan]    = tr("cyan");
  useStaticColorsTexts[magenta] = tr("magenta");

  Set();
}

void cMenuSetupMode::Set(void)
{
  int current = Current();
  Clear();

  Add(new cMenuEditStraItem(tr("Mode"), (int*)&SolarstormSetup.Mode, 2, useModeTexts));
  if (SolarstormSetup.Mode == live_picture_hsv)
  {
    Add(new cMenuEditUCharItem(tr("  Picture analysis depth [%]"), &SolarstormSetup.AnalysisDepth, 0, 50));
    Add(new cMenuEditUCharItem(tr("  Edge weighting"), &SolarstormSetup.EdgeWeighting, 1, 20));
    Add(new cMenuEditUCharItem(tr("  Min. input brightness"), &SolarstormSetup.MinInputBrightness, 0, 100));
    Add(new cMenuEditUCharItem(tr("  Min. output brightness"), &SolarstormSetup.MinOutputBrightness, 0, 100));
    Add(new cMenuEditUCharItem(tr("  Brightness [%]"), &SolarstormSetup.BrightnessCorrection, 50, 250));
    Add(new cMenuEditUCharItem(tr("  Histogram analysis width"), &SolarstormSetup.HistogramAnalysisWidth, 0, 5));
  }
  else if (SolarstormSetup.Mode == static_color)
  {
    Add(new cMenuEditStraItem(tr("  Static color"), (int*)&SolarstormSetup.StaticColor, 8, useStaticColorsTexts));
  }

  SetCurrent(Get(current));
  Display();
}

bool cMenuSetupMode::SetupParse(const char *Name, const char *Value)
{
       if (!strcasecmp(Name, MODE))                   SolarstormSetup.Mode                   = (tMode)atoi(Value);
  else if (!strcasecmp(Name, ANALYSISDEPTH))          SolarstormSetup.AnalysisDepth          = atoi(Value);
  else if (!strcasecmp(Name, EDGEWEIGHTING))          SolarstormSetup.EdgeWeighting          = atoi(Value);
  else if (!strcasecmp(Name, MININPUTBRIGHTNESS))     SolarstormSetup.MinInputBrightness     = atoi(Value);
  else if (!strcasecmp(Name, MINOUTPUTBRIGHTNESS))    SolarstormSetup.MinOutputBrightness    = atoi(Value);
  else if (!strcasecmp(Name, BRIGHTNESSCORRECTION))   SolarstormSetup.BrightnessCorrection   = atoi(Value);
  else if (!strcasecmp(Name, HISTOGRAMANALYSISWIDTH)) SolarstormSetup.HistogramAnalysisWidth = atoi(Value);
  else if (!strcasecmp(Name, STATICCOLOR))            SolarstormSetup.StaticColor            = (tStaticColors)atoi(Value);
  else return false;

  return true;
}

void cMenuSetupMode::Store(void)
{
  SolarstormPlugin->SetupStore(MODE,                   SolarstormSetup.Mode);
  SolarstormPlugin->SetupStore(ANALYSISDEPTH,          SolarstormSetup.AnalysisDepth);
  SolarstormPlugin->SetupStore(EDGEWEIGHTING,          SolarstormSetup.EdgeWeighting);
  SolarstormPlugin->SetupStore(MININPUTBRIGHTNESS,     SolarstormSetup.MinInputBrightness);
  SolarstormPlugin->SetupStore(MINOUTPUTBRIGHTNESS,    SolarstormSetup.MinOutputBrightness);
  SolarstormPlugin->SetupStore(BRIGHTNESSCORRECTION,   SolarstormSetup.BrightnessCorrection);
  SolarstormPlugin->SetupStore(HISTOGRAMANALYSISWIDTH, SolarstormSetup.HistogramAnalysisWidth);
  SolarstormPlugin->SetupStore(STATICCOLOR,            SolarstormSetup.StaticColor);
}

eOSState cMenuSetupMode::ProcessKey(eKeys Key)
{
  int prev_Mode = SolarstormSetup.Mode;
  eOSState state = cOsdMenu::ProcessKey(Key);

  if(prev_Mode != SolarstormSetup.Mode)
  {
    Set();
    state = osContinue;
  }

  if (state == osUnknown)
  {
    if ((Key == kOk) || (Key == kBack))
    {
      Store();
      state = osBack;
    }
  }
  return state;
}

// --- cMenuSetupFilter ------------------------------------------------------

#define FILTER_MODE          "FilterMode"
#define FILTER_MEANLENGTH    "FilterMeanLength"
#define FILTER_MEANTHRESHOLD "FilterMeanThreshold"
#define FILTER_PERCENTNEW    "FilterPercentNew"

cMenuSetupFilter::cMenuSetupFilter(void) : cOsdMenu(tr("Filter settings"), 25)
{
  useFilterTexts[0] = tr("off");
  useFilterTexts[1] = tr("combined");

  Set();
}

void cMenuSetupFilter::Set(void)
{
  int current = Current();
  Clear();

  Add(new cMenuEditStraItem(tr("Filter"), (int*)&SolarstormSetup.FilterMode, 2, useFilterTexts));
  if (SolarstormSetup.FilterMode == combined)
  {
    Add(new cMenuEditIntItem(tr("  Filter length [frames]"), &SolarstormSetup.FilterMeanLength, 1, 250));
    Add(new cMenuEditIntItem(tr("  Filter threshold [%]"),   &SolarstormSetup.FilterMeanThreshold, 0, 100));
    Add(new cMenuEditIntItem(tr("  Filter smoothness [%]"),  &SolarstormSetup.FilterPercentNew, 0, 100));
  }

  SetCurrent(Get(current));
  Display();
}

bool cMenuSetupFilter::SetupParse(const char *Name, const char *Value)
{
       if (!strcasecmp(Name, FILTER_MODE))          SolarstormSetup.FilterMode          = (tFilter)atoi(Value);
  else if (!strcasecmp(Name, FILTER_MEANLENGTH))    SolarstormSetup.FilterMeanLength    = atoi(Value);
  else if (!strcasecmp(Name, FILTER_MEANTHRESHOLD)) SolarstormSetup.FilterMeanThreshold = atoi(Value);
  else if (!strcasecmp(Name, FILTER_PERCENTNEW))    SolarstormSetup.FilterPercentNew    = atoi(Value);
  else return false;

  return true;
}

void cMenuSetupFilter::Store(void)
{
  SolarstormPlugin->SetupStore(FILTER_MODE,          SolarstormSetup.FilterMode);
  SolarstormPlugin->SetupStore(FILTER_MEANLENGTH,    SolarstormSetup.FilterMeanLength);
  SolarstormPlugin->SetupStore(FILTER_MEANTHRESHOLD, SolarstormSetup.FilterMeanThreshold);
  SolarstormPlugin->SetupStore(FILTER_PERCENTNEW,    SolarstormSetup.FilterPercentNew);
  ((cPluginSolarstorm*)SolarstormPlugin)->ResetFilter();
}

eOSState cMenuSetupFilter::ProcessKey(eKeys Key)
{
  int prev_FilterMode = SolarstormSetup.FilterMode;
  eOSState state = cOsdMenu::ProcessKey(Key);

  if(prev_FilterMode != SolarstormSetup.FilterMode)
  {
    Set();
    state = osContinue;
  }

  if (state == osUnknown)
  {
    if ((Key == kOk) || (Key == kBack))
    {
      Store();
      state = osBack;
    }
  }
  return state;
}

// -- cMenuSetupChannels --------------------------------------------------------

#define CHANNELS_LEFT    "ChannelsLeft"
#define CHANNELS_TOP     "ChannelsTop"
#define CHANNELS_RIGHT   "ChannelsRight"
#define CHANNELS_BOTTOM  "ChannelsBottom"

cMenuSetupChannels::cMenuSetupChannels(void) : cOsdMenu(tr("Number of channels"), 25)
{
  Set();
}

void cMenuSetupChannels::Set(void)
{
  cString buf;
  cOsdItem *item;

  int current = Current();
  Clear();

  buf = cString::sprintf("%s", tr("Please restart VDR after changing!"));
  item = new cOsdItem(*buf);
  item->SetSelectable(false);
  Add(item);

  Add(new cMenuEditUCharItem(tr("Left"),   &SolarstormSetup.ChannelsLeft, 0, 10));
  Add(new cMenuEditUCharItem(tr("Top"),    &SolarstormSetup.ChannelsTop, 0, 10));
  Add(new cMenuEditUCharItem(tr("Right"),  &SolarstormSetup.ChannelsRight, 0, 10));
  Add(new cMenuEditUCharItem(tr("Bottom"), &SolarstormSetup.ChannelsBottom, 0, 10));

  SetCurrent(Get(current));
  Display();
}

bool cMenuSetupChannels::SetupParse(const char *Name, const char *Value)
{
       if (!strcasecmp(Name, CHANNELS_LEFT))   SolarstormSetup.ChannelsLeft   = atoi(Value);
  else if (!strcasecmp(Name, CHANNELS_TOP))    SolarstormSetup.ChannelsTop    = atoi(Value);
  else if (!strcasecmp(Name, CHANNELS_RIGHT))  SolarstormSetup.ChannelsRight  = atoi(Value);
  else if (!strcasecmp(Name, CHANNELS_BOTTOM)) SolarstormSetup.ChannelsBottom = atoi(Value);
  else return false;

  return true;
}

void cMenuSetupChannels::Store(void)
{
  SolarstormPlugin->SetupStore(CHANNELS_LEFT,   SolarstormSetup.ChannelsLeft);
  SolarstormPlugin->SetupStore(CHANNELS_TOP,    SolarstormSetup.ChannelsTop);
  SolarstormPlugin->SetupStore(CHANNELS_RIGHT,  SolarstormSetup.ChannelsRight);
  SolarstormPlugin->SetupStore(CHANNELS_BOTTOM, SolarstormSetup.ChannelsBottom);
}

eOSState cMenuSetupChannels::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown)
  {
    if ((Key == kOk) || (Key == kBack))
    {
      Store();
      state = osBack;
    }
  }
  return state;
}

// --- cMenuSetupGamma -------------------------------------------------------

#define GAMMA_RED   "GammaRed"
#define GAMMA_GREEN "GammaGreen"
#define GAMMA_BLUE  "GammaBlue"

cMenuSetupGamma::cMenuSetupGamma(void) : cOsdMenu(tr("Gamma settings"), 25)
{
  Set();
}

void cMenuSetupGamma::Set(void)
{
  int current = Current();
  Clear();

  Add(new cMenuEditUCharItem(tr("Gamma red [123 => 1.23]"),   &SolarstormSetup.Gamma.r, 100, 250));
  Add(new cMenuEditUCharItem(tr("Gamma green [123 => 1.23]"), &SolarstormSetup.Gamma.g, 100, 250));
  Add(new cMenuEditUCharItem(tr("Gamma blue [123 => 1.23]"),  &SolarstormSetup.Gamma.b, 100, 250));

  SetCurrent(Get(current));
  Display();
}

bool cMenuSetupGamma::SetupParse(const char *Name, const char *Value)
{
       if (!strcasecmp(Name, GAMMA_RED))   SolarstormSetup.Gamma.r = atoi(Value);
  else if (!strcasecmp(Name, GAMMA_GREEN)) SolarstormSetup.Gamma.g = atoi(Value);
  else if (!strcasecmp(Name, GAMMA_BLUE))  SolarstormSetup.Gamma.b = atoi(Value);
  else return false;

  return true;
}

void cMenuSetupGamma::Store(void)
{
  SolarstormPlugin->SetupStore(GAMMA_RED,   SolarstormSetup.Gamma.r);
  SolarstormPlugin->SetupStore(GAMMA_GREEN, SolarstormSetup.Gamma.g);
  SolarstormPlugin->SetupStore(GAMMA_BLUE,  SolarstormSetup.Gamma.b);
  ((cPluginSolarstorm*)SolarstormPlugin)->CalcGamma();
}

eOSState cMenuSetupGamma::ProcessKey(eKeys Key)
{
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown)
  {
    if ((Key == kOk) || (Key == kBack))
    {
      Store();
      state = osBack;
    }
  }
  return state;
}

// --- cMenuEditUCharItem -----------------------------------------------------

cMenuEditUCharItem::cMenuEditUCharItem(const char *Name, unsigned char *Value, unsigned char Min, unsigned char Max)
:cMenuEditItem(Name)
{
  value = Value;
  min = Min;
  max = Max;
       if (*value < min) *value = min;
  else if (*value > max) *value = max;
  Set();
}

void cMenuEditUCharItem::Set(void)
{
  char buf[16];
  snprintf(buf, sizeof(buf), "%d", *value);
  SetValue(buf);
}

eOSState cMenuEditUCharItem::ProcessKey(eKeys Key)
{
  eOSState state = cMenuEditItem::ProcessKey(Key);

  if (state == osUnknown)
  {
    int newValue = *value;
    bool IsRepeat = Key & k_Repeat;

    Key = NORMALKEY(Key);
    switch (Key)
    {
      case kNone:
      break;

      case k0 ... k9:
           if (fresh)
           {
             newValue = 0;
             fresh = false;
           }
           newValue = newValue * 10 + (Key - k0);
      break;

      case kLeft:
           newValue = *value - 1;
           fresh = true;
           if (!IsRepeat && newValue < min && max != 255) newValue = max;
      break;

      case kRight:
           newValue = *value + 1;
           fresh = true;
           if (!IsRepeat && newValue > max && min != 0) newValue = min;
      break;

      default:
           if (*value < min) { *value = min; Set(); }
           if (*value > max) { *value = max; Set(); }
           return state;
    }
    
    if (newValue != *value && (!fresh || min <= newValue) && newValue <= max)
    {
      *value = newValue;
      Set();
    }
    state = osContinue;
  }
  return state;
}
