/*
 * setup.c: setup class
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#include <stdlib.h>

#include "defs.h"
#include "setup.h"

cSolarstormSetup::cSolarstormSetup()
{
  // Size of Captured Frame
  CaptureWidth  = 60;
  CaptureHeight = 48;

  // Channels
  ChannelsLeft   = 1;
  ChannelsTop    = 1;
  ChannelsRight  = 1;
  ChannelsBottom = 1;

  // Start Mode: always on
  StartMode = 0;

  // Display Mode: live picture
  Mode = live_picture_hsv;

  // Static Color
  StaticColor = black;

  // Calculation Parameters for live picture
  AnalysisDepth          = 20;  // depth of picture analysis (0-50%)
  EdgeWeighting          = 8;   // weighting of distance to edge
  MinInputBrightness     = 10;  // minimum brightness of pixel in input image
  MinOutputBrightness    = 0;   // minimum output brightness
  BrightnessCorrection   = 100; // brightness correction
  HistogramAnalysisWidth = 3;   // width of the histogram analysis

  // Filter
  FilterMode          = combined;
  FilterMeanLength    = 15; // length of floating mean value filter
  FilterMeanThreshold = 60; // threshold of the mean value filter
  FilterPercentNew    = 50; // filter smoothness

  // Gamma Correction
  Gamma.r = Gamma.g = Gamma.b = 220;

  // White Calibration
  // see WhiteCalibration->Init();
}
