/*
 * setup.h: setup class
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id$
 */

#ifndef _SETUP__H
#define _SETUP__H

#include <stdint.h>

class cSolarstormSetup
{
public:
  // Size of Captured Frame
  uint16_t CaptureWidth;
  uint16_t CaptureHeight;

  // Channels
  uint8_t ChannelsLeft;
  uint8_t ChannelsTop;
  uint8_t ChannelsRight;
  uint8_t ChannelsBottom;

  // Start Mode: on, off, timer dependent
  int StartMode;

  // Display Mode: live picture, static color
  tMode Mode;

  // Static Color
  tStaticColors StaticColor;

  // Calculation Parameters for live picture
  uint8_t AnalysisDepth;          // depth of picture analysis from edge (0-50%)
  uint8_t EdgeWeighting;          // weighting of distance to edge
  uint8_t MinInputBrightness;     // minimum brightness of pixel in input image
  uint8_t MinOutputBrightness;    // minimum output brightness
  uint8_t BrightnessCorrection;   // brightness correction
  uint8_t HistogramAnalysisWidth; // width of the histogram analysis

  // Filter
  tFilter FilterMode;      // filter mode: 0=off, 1=combined
  int FilterMeanLength;    // length of floating mean value filter
  int FilterMeanThreshold; // threshold of the mean value filter
  int FilterPercentNew;    // percentage of new color (for filtering)

  // Gamma Correction
  tRGBColor Gamma;

  // White Calibration
  tRGBColor *WhiteCalibration;

  cSolarstormSetup();
};
extern cSolarstormSetup SolarstormSetup;

#endif // _SETUP__H
